package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.common.StringUtils;
import com.yunzhi.demo.vo.SortParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.demo.service.ISysBannerService;
import com.yunzhi.demo.entity.SysBanner;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 广告栏 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */

@Api(tags = "广告栏")
@RestController
@RequestMapping("/")
public class SysBannerController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysBannerController.class);

    @Autowired
    public ISysBannerService iSysBannerService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/banner",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bannerList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("标题") @RequestParam(value = "title", required = false) String title) throws Exception{

        IPage<SysBanner> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<SysBanner> queryWrapper = new QueryWrapper<SysBanner>()
                .like(!StringUtils.isEmpty(title), "title", title)
                .gt("status", Constants.STATUS_DELETED)
                .orderByAsc("sort_no")
                .orderByDesc("create_date");

        IPage<SysBanner> result = iSysBannerService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysBanner 实体对象
     * @return
     */
    @RequestMapping(value="/admin/banner",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean bannerAdd(@ApiParam("保存内容") @RequestBody SysBanner sysBanner) throws Exception{

        QueryWrapper<SysBanner> queryWrapper = new QueryWrapper<SysBanner>()
                .orderByDesc("sort_no")
                .last("limit 1");

        SysBanner max = iSysBannerService.getOne(queryWrapper);
        int sortNo = max == null ? 0 : max.getSortNo() + 1;
        sysBanner.setSortNo(sortNo);

        if (iSysBannerService.save(sysBanner)){
            return ResponseBean.success(sysBanner);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/banner/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean bannerDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        SysBanner sysBanner = iSysBannerService.getById(id);
        if (null == sysBanner || Constants.STATUS_DELETED.equals(sysBanner.getStatus())) {
            return ResponseBean.success("success");
        }

        if(iSysBannerService.updateById(sysBanner)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysBanner 实体对象
     * @return
     */
    @RequestMapping(value="/admin/banner/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean bannerUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody SysBanner sysBanner) throws Exception{

        if (iSysBannerService.updateById(sysBanner)){
            return ResponseBean.success(iSysBannerService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/banner/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean bannerGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        SysBanner sysBanner = iSysBannerService.getById(id);
        if (null == sysBanner || Constants.STATUS_DELETED.equals(sysBanner.getStatus())) {
            return ResponseBean.error("未找到当前ID对应信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        return ResponseBean.success(sysBanner);
    }


    @RequestMapping(value="/admin/sort/banner",method= RequestMethod.PUT)
    @ApiOperation(value="排序", notes = "排序", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean bannerSort(@ApiParam("排序") @RequestBody SortParams sortParams) throws Exception {
        if (StringUtils.isEmpty(sortParams.getFrom()) || StringUtils.isEmpty(sortParams.getTo())) {
            return ResponseBean.success("success");
        }

        iSysBannerService.reSort(sortParams);
        return ResponseBean.success("success");
    }
}
