package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.entity.SysRole;
import com.yunzhi.demo.service.ISysRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.demo.service.ISysRoleMenuService;
import com.yunzhi.demo.entity.SysRoleMenu;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
    * 角色菜单 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */

@Api(tags = "角色菜单")
@RestController
@RequestMapping("/")
public class SysRoleMenuController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysRoleMenuController.class);

    @Autowired
    ISysRoleMenuService iSysRoleMenuService;

    @Autowired
    ISysRoleService iSysRoleService;

    /**
     * 查询角色授权菜单
     * @param roleId
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/role-menu",method= RequestMethod.GET)
    @ApiOperation(value="查询角色授权菜单", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysRoleMenuList(@ApiParam("角色ID") @RequestParam Integer roleId) throws Exception{
            QueryWrapper<SysRoleMenu> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("role_id", roleId);

            List<SysRoleMenu> roleMenuList = iSysRoleMenuService.list(queryWrapper);
            return ResponseBean.success(roleMenuList);
    }

    /**
     * 授权菜单
     * @param roleId
     * @param roleMenuList
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/role-menu/{roleId}",method= RequestMethod.POST)
    @ApiOperation(value="授权菜单", notes = "授权菜单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean sysRoleMenuAdd(@ApiParam("角色ID") @PathVariable Integer roleId,
                                       @ApiParam("保存内容") @RequestBody List<SysRoleMenu> roleMenuList) throws Exception{
        if (Constants.SUPER_ADMIN_ROLE.equals(roleId)) {
            throw new Exception("您无权进行此操作");
        }

        SysRole sysRole = iSysRoleService.getById(roleId);
        if (null == sysRole || Constants.STATUS_DELETED.equals(sysRole.getStatus())) {
            throw new Exception("验证角色信息失败: 当前角色不存在");
        }

        // 先删除所有授权信息
        iSysRoleMenuService.deleteByRole(roleId);
        if (null == roleMenuList || roleMenuList.size() == 0) {
            return ResponseBean.success("success");
        }

        for (SysRoleMenu roleMenu : roleMenuList) {
            roleMenu.setRoleId(roleId);
        }

        if (iSysRoleMenuService.saveBatch(roleMenuList)){
            return ResponseBean.success(roleMenuList);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysRoleMenu/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean sysRoleMenuDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iSysRoleMenuService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysRoleMenu 实体对象
     * @return
     */
    @RequestMapping(value="/sysRoleMenu/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean sysRoleMenuUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody SysRoleMenu sysRoleMenu) throws Exception{

        if (iSysRoleMenuService.updateById(sysRoleMenu)){
            return ResponseBean.success(iSysRoleMenuService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysRoleMenu/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysRoleMenuGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iSysRoleMenuService.getById(id));
    }
}
