package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.common.StringUtils;
import com.yunzhi.demo.entity.TaPerson;
import com.yunzhi.demo.entity.TaStudent;
import com.yunzhi.demo.service.ITaPersonService;
import com.yunzhi.demo.service.ITaStudentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.demo.service.ITaMedicalLogService;
import com.yunzhi.demo.entity.TaMedicalLog;

/**
 * <p>
    * 就诊记录 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */

@Api(tags = "就诊记录")
@RestController
@RequestMapping("/")
public class TaMedicalLogController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaMedicalLogController.class);

    @Autowired
    public ITaMedicalLogService iTaMedicalLogService;

    @Autowired
    public ITaStudentService iTaStudentService;

    @Autowired
    public ITaPersonService iTaPersonService;

    @GetMapping("/ma/medical-log")
    @ApiOperation(value="我的就诊记录", notes = "我的就诊记录", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean maMedicalList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception {
        TaPerson taPerson = getCurrentPerson();
        TaStudent taStudent = iTaStudentService.getByPersonId(taPerson.getPersonId());

        if (null == taStudent) {
            return ResponseBean.error("请先完善你的学生信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        IPage<TaMedicalLog> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaMedicalLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("student_id", taStudent.getStudentId());
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");

        IPage<TaMedicalLog> result = iTaMedicalLogService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/medicalLog",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean medicalLogList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("学校ID") @RequestParam(value ="schoolId", required = false) String schoolId,
                                       @ApiParam("专业ID") @RequestParam(value ="specialtyId", required = false) String specialtyId,
                                       @ApiParam("学号") @RequestParam(value ="studentNo", required = false) String studentNo) throws Exception{

        IPage<TaMedicalLog> pg = new Page<>(pageNum, pageSize);
        IPage<TaMedicalLog> result = iTaMedicalLogService.getMedicalLogPagedBy(pg, schoolId, specialtyId, studentNo);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taMedicalLog 实体对象
     * @return
     */
    @RequestMapping(value="/admin/medicalLog",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taMedicalLogAdd(@ApiParam("保存内容") @RequestBody TaMedicalLog taMedicalLog) throws Exception {

        String schoolId = taMedicalLog.getSchoolId();
        String studentNo = taMedicalLog.getStudentNo();

        if (StringUtils.isEmpty(schoolId) || StringUtils.isEmpty(studentNo)) {
            throw new Exception("学生信息不完整");
        }

        TaStudent taStudent = iTaStudentService.getByStudentNo(schoolId, studentNo);
        if (null == taStudent) {
            taStudent = new TaStudent();
            taStudent.setStudentNo(studentNo);
            taStudent.setSchoolId(schoolId);
            taStudent.setSpecialtyId(taMedicalLog.getSpecialtyId());
            taStudent.setName(taMedicalLog.getName());
            taStudent.setSex(taMedicalLog.getSex());
            taStudent.setPhone(taMedicalLog.getPhone());
            // 映射人员
            TaPerson taPerson = iTaPersonService.getByPhone(taStudent.getPhone());
            if (null != taPerson) {
                taStudent.setPersonId(taPerson.getPersonId());
            }

            iTaStudentService.save(taStudent);
        }

        taMedicalLog.setStudentId(taStudent.getStudentId());

        if (iTaMedicalLogService.save(taMedicalLog)){
            return ResponseBean.success(taMedicalLog);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/medical-log/{id}",method= RequestMethod.GET)
    @ApiOperation(value="就诊详情", notes = "就诊详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taMedicalLogGet(@ApiParam(value = "客户端", allowableValues = "admin,ma") @PathVariable String client,
                                        @ApiParam("就诊ID") @PathVariable Integer id) throws Exception{
        TaMedicalLog medicalLog = iTaMedicalLogService.getProfileBy(id);
        if (null == medicalLog) {
            throw new Exception("无效的就诊记录");
        }

        return ResponseBean.success(medicalLog);
    }


}
