package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.common.StringUtils;
import com.yunzhi.demo.entity.*;
import com.yunzhi.demo.service.ITaPersonDataService;
import com.yunzhi.demo.service.ITaStudentService;
import com.yunzhi.demo.service.ITdSchoolService;
import com.yunzhi.demo.service.ITdSpecialtyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

@Slf4j
@Api(tags = "学生")
@RestController
@RequestMapping("/")
public class TaStudentController extends BaseController {

    @Autowired
    ITaStudentService iTaStudentService;

    @Autowired
    ITaPersonDataService iTaPersonDataService;

    @Autowired
    ITdSchoolService iTdSchoolService;

    @Autowired
    ITdSpecialtyService iTdSpecialtyService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/student",method= RequestMethod.GET)
    @ApiOperation(value="查询学生列表", notes = "查询学生列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taStudentList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                     @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("名称") @RequestParam(value ="name", required = false) String name,
                                     @ApiParam("学校ID") @RequestParam(value ="schoolId", required = false) String schoolId,
                                     @ApiParam("专业ID") @RequestParam(value ="specialtyId", required = false) String specialtyId,
                                     @ApiParam("手机") @RequestParam(value ="phone", required = false) String phone,
                                     @ApiParam("学号") @RequestParam(value ="studentNo", required = false) String studentNo) throws Exception {

        IPage<TaStudent> pg = new Page<>(pageNum, pageSize);
        IPage<TaStudent> result = iTaStudentService.getStudentInfoPagedBy(pg, name, schoolId, specialtyId, phone, studentNo);
        return ResponseBean.success(result);
    }

    /**
     * 小程序完善信息
     * @param taStudent 实体对象
     * @return
     */
    @RequestMapping(value="/ma/student",method= RequestMethod.PUT)
    @ApiOperation(value="小程序完善信息", notes = "更新当前人自己的信息", httpMethod = "PUT", response = TaStudent.class)
    public ResponseBean taPersonUpdate(@ApiParam("更新内容") @RequestBody TaStudent taStudent) throws Exception{
        TaPerson currentPerson = getCurrentPerson();
        if (null == currentPerson || null == taStudent) {
            throw new Exception("校验人员信息失败");
        }

        if (StringUtils.isEmpty(taStudent.getName())) {
            throw new Exception("姓名不能为空");
        }

        if (StringUtils.isEmpty(taStudent.getPhone())) {
            throw new Exception("手机号不能为空");
        }

        if (StringUtils.isEmpty(taStudent.getSchoolId())) {
            throw new Exception("学校不能为空");
        }

        if (StringUtils.isEmpty(taStudent.getSpecialtyId())) {
            throw new Exception("专业不能为空");
        }

        if (StringUtils.isEmpty(taStudent.getStudentNo())) {
            throw new Exception("学号不能为空");
        }

        // 解决人员跟学生映射问题
        if (StringUtils.isEmpty(taStudent.getStudentId())) {
            TaStudent origin = iTaStudentService.getByPersonId(currentPerson.getPersonId());
            if (null != origin) {
                taStudent.setStudentId(origin.getStudentId());
            } else {
                origin = iTaStudentService.getByPhone(currentPerson.getPhone());
                if (null != origin) {
                    taStudent.setStudentId(origin.getStudentId());
                }
            }
        }

        if (!StringUtils.isEmpty(taStudent.getStudentNo())) {
            TaStudent stu = iTaStudentService.getByStudentNo(taStudent.getSchoolId(), taStudent.getStudentNo());
            if (null != stu && !StringUtils.isEmpty(taStudent.getStudentId()) && !stu.getStudentId().equals(taStudent.getStudentId())) {
                throw new Exception("学号信息重复");
            }
        }

        taStudent.setUpdateDate(LocalDateTime.now());
        taStudent.setPersonId(currentPerson.getPersonId());

        if (StringUtils.isEmpty(taStudent.getStudentId())) {
            iTaStudentService.save(taStudent);
        } else {
            iTaStudentService.updateById(taStudent);
        }

        fillInfo(taStudent);
        return ResponseBean.success(taStudent);
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/student/{id}",method= RequestMethod.GET)
    @ApiOperation(value="学生详情", notes = "查询人员详细信息", httpMethod = "GET", response = TaPerson.class)
    public ResponseBean taPersonGet(@ApiParam("学生ID") @PathVariable String id) throws Exception{
        TaStudent taStudent = iTaStudentService.getById(id);
        if (null == taStudent || Constants.STATUS_DELETED.equals(taStudent.getStatus())) {
            throw new Exception("验证人员信息失败, 请退出重试");
        }

        fillInfo(taStudent);

        return ResponseBean.success(taStudent);
    }

    /**
     * 根据id查询对象
     * @param id  学号
     */
    @RequestMapping(value="/admin/student-no/{id}",method= RequestMethod.GET)
    @ApiOperation(value="学生详情", notes = "查询人员详细信息", httpMethod = "GET", response = TaPerson.class)
    public ResponseBean studentGet(@ApiParam("学生学号") @PathVariable String id,
                                   @ApiParam("学校ID") @RequestParam String schoolId) throws Exception{
        TaStudent taStudent = iTaStudentService.getByStudentNo(schoolId, id);
        if (null == taStudent || Constants.STATUS_DELETED.equals(taStudent.getStatus())) {
            throw new Exception("验证人员信息失败, 请退出重试");
        }

        fillInfo(taStudent);

        return ResponseBean.success(taStudent);
    }

    @GetMapping("/ma/currentPerson")
    @ApiOperation(value="小程序当前人员", notes = "获取当前人员信息", httpMethod = "GET", response = TaPerson.class)
    public ResponseBean getCurrent() throws Exception {
        TaPerson taPerson = getCurrentPerson();
        if (null == taPerson) {
            throw new Exception("验证人员信息失败, 请退出重试");
        }

        TaPersonData taPersonData = iTaPersonDataService.getById(taPerson.getPersonId());
        taPerson.setPersonData(taPersonData);

        // 先找映射ID的
        TaStudent taStudent = iTaStudentService.getByPersonId(taPerson.getPersonId());
        if (null == taStudent) {
            // 如果没有再找相同手机号的
            if (!StringUtils.isEmpty(taPerson.getPhone())) {
                taStudent = iTaStudentService.getByPhone(taPerson.getPhone());
                if (null != taStudent) {
                    if (!StringUtils.isEmpty(taStudent.getPersonId())) {
                        log.error("学生[{}]与人员[{}]映射异常", taStudent.getStudentId(), taPerson.getPersonId());
                    } else {
                        taStudent.setPersonId(taPerson.getPersonId());
                        iTaStudentService.updateById(taStudent);
                    }
                }
            }
        }

        if (null != taStudent) {
            fillInfo(taStudent);
        }

        taPerson.setStudent(taStudent);

        return ResponseBean.success(taPerson);
    }

    private void fillInfo(TaStudent taStudent) {
        // 冗余学校名称, 专业名称
        if (null != taStudent.getSchoolId()) {
            TdSchool tdSchool = iTdSchoolService.getById(taStudent.getSchoolId());
            if (null != tdSchool) {
                taStudent.setSchoolName(tdSchool.getName());
            }

            TdSpecialty tdSpecialty = iTdSpecialtyService.getById(taStudent.getSpecialtyId());
            if (null != tdSpecialty) {
                taStudent.setSpecialtyName(tdSpecialty.getName());
            }
        }

        // 获取统计数据
        TaPersonData taPersonData = iTaPersonDataService.getById(taStudent.getPersonId());
        taStudent.setPersonData(taPersonData);
    }
}
