package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.*;
import com.yunzhi.demo.entity.*;
import com.yunzhi.demo.service.ITaStudentService;
import com.yunzhi.demo.service.ITmTestPdfService;
import com.yunzhi.demo.service.ITmTestStudentService;
import com.yunzhi.demo.vo.StudentTestLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.demo.service.ITaTestLogService;

import java.util.List;


/**
 * <p>
    * 体检记录 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */

@Api(tags = "体检记录")
@RestController
@RequestMapping("/")
public class TaTestLogController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaTestLogController.class);

    @Autowired
    public ITaTestLogService iTaTestLogService;

    @Autowired
    public ITaStudentService iTaStudentService;

    @Autowired
    public ITmTestPdfService iTmTestPdfService;

    @Autowired
    public ITmTestStudentService iTmTestStudentService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/ma/test-log",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taTestLogList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
        TaPerson taPerson = getCurrentPerson();
        TaStudent taStudent = iTaStudentService.getByPersonId(taPerson.getPersonId());
        String schoolId = taStudent == null ? "******" : taStudent.getSchoolId();
        String studentNo = taStudent == null ? "******" : taStudent.getStudentNo();

        IPage<TaTestLog> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaTestLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_id", schoolId);
        queryWrapper.eq("student_no", studentNo);
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");

        IPage<TaTestLog> result = iTaTestLogService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/testLog",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean testLogList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("学校") @RequestParam(value ="schoolId", required = false) String schoolId,
                                    @ApiParam("专业") @RequestParam(value ="specialtyId", required = false) String specialtyId,
                                    @ApiParam("学号") @RequestParam(value ="studentNo", required = false) String studentNo) throws Exception{

        IPage<StudentTestLog> pg = new Page<>(pageNum, pageSize);

        IPage<StudentTestLog> result = iTaTestLogService.queryWithStudent(pg, schoolId, specialtyId, studentNo);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taTestLog 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taTestLog",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taTestLogAdd(@ApiParam("保存内容") @RequestBody TaTestLog taTestLog) throws Exception{

        if (StringUtils.isEmpty(taTestLog.getSchoolId())) {
            return ResponseBean.error("没有找到学校信息", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (iTaTestLogService.save(taTestLog)){
            return ResponseBean.success(taTestLog);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     *
     * @param testPdfList
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/taTestLog/upload/pdf",method= RequestMethod.POST)
    @ApiOperation(value="上传体检文件", notes = "上传体检文件", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean uploadPDF(@RequestBody List<TmTestPdf> testPdfList) throws Exception {
        if (null == testPdfList || testPdfList.size() < 1) {
            return ResponseBean.error("请设置上传内容", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        iTmTestPdfService.saveBatch(testPdfList);

        return ResponseBean.success(null);
    }

    /**
     *
     * @param testStudentList
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/taTestLog/upload/excel",method= RequestMethod.POST)
    @ApiOperation(value="上传体检映射文件", notes = "上传体检映射文件", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean uploadExcel(@RequestBody List<TmTestStudent> testStudentList) throws Exception {
        if (null == testStudentList || testStudentList.size() < 1) {
            return ResponseBean.error("请设置上传内容", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        iTmTestStudentService.saveBatch(testStudentList);

        return ResponseBean.success(null);
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taTestLog/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taTestLogDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iTaTestLogService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taTestLog 实体对象
     * @return
     */
    @RequestMapping(value="/taTestLog/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taTestLogUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody TaTestLog taTestLog) throws Exception{

        if (StringUtils.isEmpty(taTestLog.getSchoolId())) {
            return ResponseBean.error("没有找到学校信息", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (iTaTestLogService.updateById(taTestLog)){
            return ResponseBean.success(iTaTestLogService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taTestLog/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taTestLogGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaTestLogService.getById(id));
    }
}
