package com.yunzhi.demo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.StringUtils;
import com.yunzhi.demo.entity.TaPostTest;
import com.yunzhi.demo.mapper.TaPostTestMapper;
import com.yunzhi.demo.service.ITaPostTestService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 题库 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */
@Service
public class TaPostTestServiceImpl extends ServiceImpl<TaPostTestMapper, TaPostTest> implements ITaPostTestService {

    @Autowired
    TaPostTestMapper taPostTestMapper;

    @Override
    public List<TaPostTest> getListByPostId(String postId) {
        QueryWrapper<TaPostTest> queryWrapper = new QueryWrapper<TaPostTest>()
                .eq("post_id", postId)
                .gt("status", Constants.STATUS_DELETED)
                .orderByDesc("create_date");
        return list(queryWrapper);
    }

    @Override
    public TaPostTest getRandRowBy(String postId) {
        return taPostTestMapper.getRandRowBy(postId);
    }

    /**
     * 判断是否答题正确
     * @param target
     * @param answer
     */
    @Override
    public boolean checkAnswer(String target, String answer) {
        List<String> correctAnswers = Arrays.asList(target.split(","));
        List<String> myAnswers = Arrays.asList(answer.split(","));

        if (correctAnswers.size() != myAnswers.size()) {
            return false;
        }

        return myAnswers.containsAll(correctAnswers);
    }
}
